define("widgets/BreadcrumbsTrail", [
 "plugins/text!widgets/templates/BreadcrumbsTrail.html"
], function(template) {
    return function(args) {
        // array of object titles
        breadcrumbsTitles = [];
        // array of dom nodes
        breadcrumbsNodes = [];
        /* the ellipses node and width to be used through the lifetime of the BreadcrumbsTrail widget */
        ellipsesNode = jQuery("<li>...</li>");
        ellipsesNodeWidth = 0;
        visibleNodesWidth = 0;
        /* Start of Utility Functions */
        var isString = function(arg) {
            if (jQuery.type(arg) === "string") {
                return true;
            }
            return false;
        };
        /*
         * used to stop event from firing multiple times for a single event.
         */
        var debounce = function(func, delay) {
            // timeout variable shared for each call to function passed to debounce
            // keeps func from being called multiple times for a continous event
            var timeout;
            delay = delay ? delay : 100; // default to 100ms
            return function() {
                var context = this;
                var args = arguments;
                /*
                 * clearing timeout here means that if the function is called multiple times
                 * before the specified/default delay it will only execute once
                 */
                clearTimeout(timeout);
                timeout = setTimeout(function() {
                    clearTimeout(timeout);
                    func.apply(func, context, args);
                }, delay);
            }
        };
        /* End Utility Functions */
        var breadcrumbsTrail = {
            domNode: null,
			treeController: null,
            init: function() {
                jQuery.extend(this, args);
                var _this = this;
                /*
                 * if the window is resized or the sidemenu is displayed/hidden
                 * re-render the breadcrumbs to display at the appropriate size
                 * for the available space.
                 */
                jQuery(window)
                    .bind("resize", debounce(jQuery.proxy(this.renderBreadcrumbs, this)));
                jQuery(window)
                    .bind("sideMenuToggled", jQuery.proxy(function() {
                        setTimeout(jQuery.proxy(this.renderBreadcrumbs, this), 250)
                    }, this));
                var breadcrumbsTrailTemplate = jQuery(template)
                    .clone();
                this.domNode = breadcrumbsTrailTemplate.get();
                // get the width of the ellipses node to be used if the breadcrumbs become too wide
                this.hideBreadcrumbNode(ellipsesNode);
                jQuery(this.domNode)
                    .append(ellipsesNode);
                ellipsesNodeWith = jQuery(ellipsesNode)
                    .width();
                var vc = ViewerController;
                vc.events.navigate.subscribe(this.updateBreadcrumbsOnNavigate, this, true);
            },
            setTreeController: function(treeController) {
                this.treeController = treeController;
            },
            updateBreadcrumbsOnNavigate: function(type, args) {
                var obj = args[0];
                var objArray = [];
                while (obj != null) {
                    objArray.unshift(obj);
                    obj = obj.parent;
                }
                this.addBreadcrumbs(objArray);
            },
            hideBreadcrumbNode: function(node) {
                jQuery(node)
                    .removeClass("breadcrumb-visible");
                jQuery(node)
                    .addClass("breadcrumb-hidden");
            },
            showBreadcrumbNode: function(node) {
                jQuery(node)
                    .removeClass("breadcrumb-hidden");
                jQuery(node)
                    .addClass("breadcrumb-visible");
            },
            /*
             * re-renders the existing breadcrumbs
             */
            renderBreadcrumbs: function() {
                var titlesBackup = breadcrumbsTitles;
                this.clearBreadcrumbs();
                this.addBreadcrumbs(titlesBackup);
            },
            breadCrumbClickedCallback: function(obj) {
				//console.log("breadcrumb clicked:  " + obj.label );
				this.treeController.nodeIconClicked(obj);
            },
            // add breadcrumb to existing breadcrumbs
            addBreadcrumb: function(obj) {
				var title = obj.label;
                if (!isString(title)) {
                    return;
                }
                breadcrumbsTitles.push(obj);
                // generate new list item and push into breadcrumbsNodes
                var breadcrumb = jQuery("<li>" + title + "</li>")
				
				breadcrumb.bind("click", jQuery.proxy(function(e) {
					this.breadCrumbClickedCallback(obj);
                }, this));

                breadcrumbsNodes.push(breadcrumb);
                // Get width of new entry to determine need for hiding earlier breadcrumbs
                this.hideBreadcrumbNode(breadcrumb);
                jQuery(this.domNode)
                    .append(breadcrumb);
                var breadcrumbWidth = jQuery(breadcrumb)
                    .width();
                visibleNodesWidth += breadcrumbWidth;
                /*
                 * Calculate the width of the breadcrumb nodes and determine if too
                 * long to be displayed on a single line.
                 */
                if (visibleNodesWidth > jQuery(this.domNode)
                    .width()) {
                    var visibleNodeIndex = 0;
                    /*
                     * Find the first visible node
                     */
                    while (visibleNodeIndex < breadcrumbsNodes.length) {
                        if (jQuery(breadcrumbsNodes[visibleNodeIndex])
                            .hasClass("breadcrumb-visible")) {
                            break; // found first visible node
                        }
                        visibleNodeIndex++;
                    }
                    /*
                     * Hide the earliest breadcrumb nodes
                     */
                    while (visibleNodesWidth + ellipsesNodeWidth > jQuery(this.domNode)
                        .width() && visibleNodeIndex < breadcrumbsNodes.length - 1) {
                        var visibleNode = jQuery(breadcrumbsNodes[visibleNodeIndex])
                        visibleNodesWidth -= visibleNode.width();
                        this.hideBreadcrumbNode(visibleNode);
                        jQuery(visibleNode)
                            .detach();
                        visibleNodeIndex++;
                    }
                    // prepend the ellipses
                    jQuery(this.domNode)
                        .prepend(ellipsesNode);
                    visibleNodesWidth += ellipsesNode.width();
                    this.showBreadcrumbNode(ellipsesNode);
                }
                // make the new breadcrumb visible
                this.showBreadcrumbNode(breadcrumb);
            },
            /* Replace existing breadcrumbs with new breadcrumbs */
            addBreadcrumbs: function(objArray) {
                this.clearBreadcrumbs();
                for (var i = 0; i < objArray.length; i++) {
                    this.addBreadcrumb(objArray[i]);
                }
            },
            /* Remove the last breadcrumb in the list of breadcrumbs */
            removeLastBreadcrumb: function() {
                var lastBreadcrumb = breadcrumbsNodes[breadcrumbsNodes.length - 1];
                visibleNodesWidth -= jQuery(breadcrumbsNodes[breadcrumbsNodes.length - 1])
                    .width();
                breadcrumbsNodes.splice(breadcrumbsNodes.length - 1, 1);
                breadcrumbsTitles.splice(breadcrumbsNodes.length - 1, 1);
                jQuery(lastbreadcrumb)
                    .remove();
            },
            clearBreadcrumbs: function() {
                jQuery(this.domNode)
                    .empty();
                this.hideBreadcrumbNode(ellipsesNode);
                breadcrumbsTitles = [];
                breadcrumbsNodes = [];
                visibleNodesWidth = 0;
            }
        };
        jQuery(breadcrumbsTrail.init());
        return breadcrumbsTrail;
    };
});

